# Ecrivons une application python dockerisable
![category](développement)
![tag](python)
![tag](docker)

L'objectif de cette note est de décrire une méthode permattant l'écriture
d'une application **python** qu'on place dans une image docker.

Toutes nos images seront basées sur l'image officiel python qu'on trouve
sur docker hub.

Notre projet aura toujours la structure suivante

![201606120928_1.png](./upload/201606120928_1.png)

Notre fichier **entrypoint.sh** est le suivant

```bash
#!/bin/bash
set -e

if [ "$1" = 'app' ]; then
    python /app/app.py
    exit
fi

exec "$@"
```

Ce fichier permet notamenent de ce connecter à l'image une fois en route via
la commande

```bash
docker exec -it MyImage bash
```

Une application simple
----------------------

Notre  fichier **Dockerfile**

```bash
FROM python:3.4-wheezy

COPY ./src/entrypoint.sh /entrypoint.sh
RUN chmod +x /entrypoint.sh

RUN mkdir /app
COPY ./src/app.py /app/app.py

ENTRYPOINT ["/entrypoint.sh"]
CMD ["app"]
```

Notre fichier **app.py**

```python
#!/usr/bin/env python
# -*- coding: utf-8 -*-
print("hello world")
```

l'utilisation

```bash
docker build -t test .
docker run --name test test
```

Cette application est une application qui ne tourne pas ... elle écrit hello world
puis s'arrête. Nous avons souvent besoin d'application qui tourne sur elle même.

Une application Flask
---------------------

Notre application répondra à l'url http://127.0.0.1:5000/

Notre  fichier **Dockerfile**

```bash
FROM python:3.4-wheezy

COPY ./src/entrypoint.sh /entrypoint.sh
RUN chmod +x /entrypoint.sh

RUN pip install flask

RUN mkdir /app
COPY ./src/app.py /app/app.py

EXPOSE 5000

ENTRYPOINT ["/entrypoint.sh"]
CMD ["app"]
```

Notre fichier **app.py**

```python
#!/usr/bin/env python
# -*- coding: utf-8 -*-
from flask import Flask
import json

app = Flask(__name__)

@app.route("/")
def hello():
    return json.dumps({"msg": "Hello World!"})

if __name__ == "__main__":
    app.run(host="0.0.0.0", port="5000")
```

l'utilisation

```bash
docker build -t test .
docker run --name test -p 5000:5000 test
```

Une application RabbitMq
------------------------

Notre application lit les messages d'une queue et les copies dans une autre queue.

Notre  fichier **Dockerfile**

```bash
FROM python:3.4-wheezy

COPY ./src/entrypoint.sh /entrypoint.sh
RUN chmod +x /entrypoint.sh

RUN pip install pika

RUN mkdir /app
COPY ./src/app.py /app/app.py


ENTRYPOINT ["/entrypoint.sh"]
CMD ["app"]
```

Notre fichier **app.py**

```python
#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pika

connection = pika.BlockingConnection(
    pika.ConnectionParameters(
        host='192.168.1.19',
        credentials = pika.PlainCredentials('guest', 'guest')
        )
    )
channel = connection.channel()

def callback(ch, method, properties, body):
    channel.basic_publish(exchange='',
                          routing_key='QueueTestCopy',
                          body= body)

channel.basic_consume(callback,
                  queue='QueueTest',
                  no_ack=True)

channel.start_consuming()
```

l'utilisation

```bash
docker build -t test .
docker run --name test test
```

Si cette application fonctionne bien, son principale problème est le manque de 
configuration ... le prochain chapitre gère ce problème

Utilisation de paramètre passé par docker
-----------------------------------------

Docker permet le passage d'information via l'environnement de l'image.

Je propose de reprendre le dernier exemple et de le rendre plys dynamique.

Le code python devient

```python
#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pika
import os

QueueIn = os.environ['QUEUE_IN']
QueueOut = os.environ['QUEUE_OUT']
MQ_user = os.environ['MQ_USER']
MQ_password = os.environ['MQ_PASSWORD']
MQ_host = os.environ['MQ_HOST']

connection = pika.BlockingConnection(
    pika.ConnectionParameters(
        host= MQ_host,
        credentials = pika.PlainCredentials(MQ_user, MQ_password)
        )
    )
channel = connection.channel()

def callback(ch, method, properties, body):
    channel.basic_publish(exchange='',
                          routing_key= QueueOut,
                          body= body)

channel.basic_consume(callback,
                  queue= QueueIn,
                  no_ack=True)

channel.start_consuming()
```

Puis on lance notre images avec des informations d'environnements

```bash
docker build -t test .
docker run --name test -e QUEUE_IN=QueueTest -e QUEUE_OUT=QueueTestCopy -e MQ_USER=guest -e MQ_PASSWORD=guest -e MQ_HOST=192.168.1.19 test
```

Sous docker-compose le passage d'environnement est réalisé de la façon suivante

```
test:
    build: test/. 
    environment:
        QUEUE_IN : QueueTest
        QUEUE_OUT : QueueTestCopy
        MQ_USER : guest
        MQ_PASSWORD : guest
        MQ_HOST : 192.168.1.19
```

```warning
je déconseille l'utilisation de variable d'environnement dans une application
classique .. mais pour docker je vois cela comme un fichier de confiuration
n'impactant pas ma machine locale
```

```note
si on souhaite connaitre l'IP de sa machine locale et l'utiliser ensuite dans 
le docker-compose, on peut ajouter dans son .bashrc une variable donnant cette
information

.. code-block:: bash

    export MYIP=$(ifconfig eth0 | grep "inet adr" | cut -d ':' -f 2 | cut -d ' ' -f 1) 

et l'utilisation dans docker-compose

```
MQ_HOST : $MYIP``````
